/*
www.E2CRE8.be - Brainbox Arduino  - by Bart Huyskens
20/02/2016

FULL DUPLEX communication bewteen the BBA and a Android smartdevice running a selfmade APPINVENTOR APP
We use the HC06 Bluetooth device (3€ in CHina)
The APP sourcecode and installer are both included in this project and are free to use and alter 

BE AWARE:
The Bluetooth data received from the smartdevice to the uC is not time consistent. 
It may take up to 100msec before the first byte is received and delays up to 75msec between bytes are common
This implies that programming the uC without interrupts will take some extra measures
we do this by:
- waiting long enough before we stop waiting for incoming data (buit in in arduino instructions)
- using a testbyte (dec85) that needs to come in first before we display the next bytes

THIS program sends 7 8-bit byte variables 
- the first byte is a testbyte and is always 0xAA
- The 6 other bytes are from the 6 A/D channels of the BBA 
- via RS232 & the HC06 Bluetooth device 
- to a Android APP (made in app inventor)
After sending is finished it waits for the APP to send 5 byte vaiables back
- the first byte is a testbyte and is always 0x55(Dec 85)
- the other 4 bytes are from 4 sliders in the app
- these 4 bytes are displayed at the I2C LCD

Connection of HC06 Bluetooth device to Brainbox ARduino:
!!use the 4 pin connector labled 'For HC06' - this fixes all connections for you.
BBA - pin TX    to HC06 pin RXD
BBA - pin RX    to HC06 pin TXD
BBA - +5V       to HC06 pin VCC
BBA - GND       to HC06 pin GND

This program drives an I2C LCD of the type: 
 16 character 2 line I2C Display
 Backpack Interface labelled "YwRobot Arduino LCM1602 IIC V1"  (2€ @ aliexpress)

Connect this LCD as follows:
LCD             Brainbox Arduino
GND             GND
VCC             +5V
SDA             SDA/2
SCL             SCL/3
!! Pull up resistors are optional and not required because ARduino wire lib activates the weak internal pull ups of an AVR - place 4K7 between SDA and 5V and 4K7 between SCL and 5V

To communicate correctly with this I2C LCD you need to install the <LiquidCrystal_I2C.h> library in the arduino IDE
  1- download the "LiquidCrystal_I2C" library as a zip file from https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
  2- do not unzip the file
  3- in ARduino IDE: Sketch >> include library >> add .ZIP library - select the downloaded zip file
  4- this library is installed under 'mydocs'->Arduino : remove it by deleting it there
*/

#include <Wire.h>  // Comes with Arduino IDE
#include <LiquidCrystal_I2C.h> // Use the extra I2C LCD library: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
  

//Declare the AD variables as int because Arduino always executes a 10 bit AD conversion
int AD_A0 = 0;  
int AD_A1 = 0;  
int AD_A2 = 0;  
int AD_A3 = 0;  
int AD_A4 = 0;  
int AD_A5 = 0;  

// These 8 bit Byte variables are used to downsize the 10 bit AD conversion result into 8 bit
byte ADC_A0 = 0;  
byte ADC_A1 = 0;  
byte ADC_A2 = 0;  
byte ADC_A3 = 0; 
byte ADC_A4 = 0;  
byte ADC_A5 = 0; 

// These 5 byte variables are used to buffer the 5 received bytes. 
//These vars need to be of the byte type because we want numbers 
//(declaring them as char would result in displaying characters)
byte RX0 = 0;
byte RX1 = 0;
byte RX2 = 0;
byte RX3 = 0;
byte RX4 = 0;



// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address


void setup()
{
  Serial1.begin(9600); //initialise RS232 comms on TX & RX pin to communicate with HC06 Bluetooth module
  lcd.begin(16,2);// initialize LCD library
  
  lcd.backlight();              //backlight on
  lcd.setCursor(0,0);           // set cursor to positon x=0, y=0
  lcd.print("TEST BT HC06");    // print text on the LCD
}
 
 
void loop()
{
AD_A0 = analogRead(A0);              // read the analog value measured at analog pin ANx 
AD_A1 = analogRead(A1);              // read the analog value measured at analog pin ANx 
AD_A2 = analogRead(A2);              // read the analog value measured at analog pin ANx 
AD_A3 = analogRead(A3);              // read the analog value measured at analog pin ANx 
AD_A4 = analogRead(A4);              // read the analog value measured at analog pin ANx 
AD_A5 = analogRead(A5);              // read the analog value measured at analog pin ANx 

ADC_A0 = map(AD_A0, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A1 = map(AD_A1, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A2 = map(AD_A2, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A3 = map(AD_A3, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A4 = map(AD_A4, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)
ADC_A5 = map(AD_A5, 0, 1023, 0, 255);    // rescale the sensorValue (0-1024) to (0-255)

Serial1.write(0xAA);    // send flow control byte
Serial1.write(ADC_A0);  // send byte to SMartdevice over RS232 / HC06
Serial1.write(ADC_A1);  // send byte to SMartdevice over RS232 / HC06
Serial1.write(ADC_A2);  // send byte to SMartdevice over RS232 / HC06
Serial1.write(ADC_A3);  // send byte to SMartdevice over RS232 / HC06
Serial1.write(ADC_A4);  // send byte to SMartdevice over RS232 / HC06
Serial1.write(ADC_A5);  // send byte to SMartdevice over RS232 / HC06



if (Serial1.available() > 0) 
    {
    // read the 5 incoming bytes from app to uC:
    RX0 = Serial1.read();
    RX1 = Serial1.read();
    RX2 = Serial1.read();
    RX3 = Serial1.read();
    RX4 = Serial1.read();
    }

if (RX0 == 85)

    {// if first byte received (flow control check byte) is not 85, do not display rest of bytes
    lcd.clear();
    lcd.setCursor(0,0); 
    lcd.print("RECEIVING DATA");
    
    lcd.setCursor(0,1);
    lcd.print(RX1); 
    lcd.setCursor(4,1);
    lcd.print(RX2);
    lcd.setCursor(8,1);
    lcd.print(RX3);
    lcd.setCursor(12,1);
    lcd.print(RX4);
    }


delay(300); // 300 msec delay because the APP only checks for new incoming data every 100msec
 
}
